from typing import Any, Sequence

from ..request import Request
from .base import BaseFunction, api_function

__all__ = ("Resource",)


class Resource(BaseFunction):
    """
    Provides interactions with resource.
    """

    @api_function
    @classmethod
    async def list(cls):
        """
        Lists all resource presets.
        """
        rqst = Request("GET", "/resource/presets")
        async with rqst.fetch() as resp:
            return await resp.json()

    @api_function
    @classmethod
    async def check_presets(cls, group: str | None = None, scaling_group: str | None = None) -> Any:
        """
        Lists all resource presets in the current scaling group with additional
        information.
        """
        rqst = Request("POST", "/resource/check-presets")
        data = {}
        if group is not None:
            data["group"] = group
        if scaling_group is not None:
            data["scaling_group"] = scaling_group
        rqst.set_json(data)
        async with rqst.fetch() as resp:
            return await resp.json()

    @api_function
    @classmethod
    async def get_docker_registries(cls):
        """
        Lists all registered docker registries.
        """
        rqst = Request("GET", "/config/docker-registries")
        async with rqst.fetch() as resp:
            return await resp.json()

    @api_function
    @classmethod
    async def usage_per_month(cls, month: str, group_ids: Sequence[str]):
        """
        Get usage statistics for groups specified by `group_ids` at specific `month`.

        :param month: The month you want to get the statistics (yyyymm).
        :param group_ids: Groups IDs to be included in the result.
        """
        rqst = Request("GET", "/resource/usage/month")
        rqst.set_json({
            "month": month,
            "group_ids": group_ids,
        })
        async with rqst.fetch() as resp:
            return await resp.json()

    @api_function
    @classmethod
    async def usage_per_period(cls, group_id: str, start_date: str, end_date: str):
        """
        Get usage statistics for a group specified by `group_id` for time between
        `start_date` and `end_date`.

        :param start_date: start date in string format (yyyymmdd).
        :param end_date: end date in string format (yyyymmdd).
        :param group_id: Groups ID to list usage statistics.
        """
        rqst = Request("GET", "/resource/usage/period")
        rqst.set_json({
            "group_id": group_id,
            "start_date": start_date,
            "end_date": end_date,
        })
        async with rqst.fetch() as resp:
            return await resp.json()

    @api_function
    @classmethod
    async def get_resource_slots(cls):
        """
        Get supported resource slots of Backend.AI server.
        """
        rqst = Request("GET", "/config/resource-slots")
        async with rqst.fetch() as resp:
            return await resp.json()

    @api_function
    @classmethod
    async def get_vfolder_types(cls):
        rqst = Request("GET", "/config/vfolder-types")
        async with rqst.fetch() as resp:
            return await resp.json()

    @api_function
    @classmethod
    async def recalculate_usage(cls):
        rqst = Request("POST", "/resource/recalculate-usage")
        async with rqst.fetch() as resp:
            return await resp.json()

    @api_function
    @classmethod
    async def user_monthly_stats(cls):
        rqst = Request("GET", "/resource/stats/user/month")
        async with rqst.fetch() as resp:
            return await resp.json()

    @api_function
    @classmethod
    async def admin_monthly_stats(cls):
        rqst = Request("GET", "/resource/stats/admin/month")
        async with rqst.fetch() as resp:
            return await resp.json()



from typing import Any, List, Mapping

from ..request import Request
from ..session import api_session
from .base import BaseFunction, api_function

__all__ = ("SessionTemplate",)


class SessionTemplate(BaseFunction):
    @api_function
    @classmethod
    async def create(
        cls,
        template: str,
        domain_name: str = None,
        group_name: str = None,
        owner_access_key: str = None,
    ) -> "SessionTemplate":
        rqst = Request("POST", "/template/session")
        if domain_name is None:
            # Even if config.domain is None, it can be guessed in the manager by user information.
            domain_name = api_session.get().config.domain
        if group_name is None:
            group_name = api_session.get().config.group
        body = {
            "payload": template,
            "group_name": group_name,
            "domain_name": domain_name,
            "owner_access_key": owner_access_key,
        }
        rqst.set_json(body)
        async with rqst.fetch() as resp:
            response = await resp.json()
            template_id = ", ".join(data["id"] for data in response)
            return cls(template_id, owner_access_key=owner_access_key)

    @api_function
    @classmethod
    async def list_templates(cls, list_all: bool = False) -> List[Mapping[str, str]]:
        rqst = Request("GET", "/template/session")
        rqst.set_json({"all": list_all})
        async with rqst.fetch() as resp:
            return await resp.json()

    def __init__(self, template_id: str, owner_access_key: str = None):
        self.template_id = template_id
        self.owner_access_key = owner_access_key

    @api_function
    async def get(self, body_format: str = "yaml") -> str:
        params = {"format": body_format}
        if self.owner_access_key:
            params["owner_access_key"] = self.owner_access_key
        rqst = Request("GET", f"/template/session/{self.template_id}", params=params)
        async with rqst.fetch() as resp:
            data = await resp.text()
        return data

    @api_function
    async def put(self, template: str) -> Any:
        body = {
            "payload": template,
        }
        if self.owner_access_key:
            body["owner_access_key"] = self.owner_access_key
        rqst = Request("PUT", f"/template/session/{self.template_id}")
        rqst.set_json(body)
        async with rqst.fetch() as resp:
            return await resp.json()

    @api_function
    async def delete(self) -> Any:
        params = {}
        if self.owner_access_key:
            params["owner_access_key"] = self.owner_access_key
        rqst = Request("DELETE", f"/template/session/{self.template_id}", params=params)
        async with rqst.fetch() as resp:
            return await resp.json()



from typing import Any, Iterable, Sequence

from ...cli.types import Undefined, undefined
from ..output.fields import keypair_resource_policy_fields
from ..output.types import FieldSpec
from ..session import api_session
from ..types import set_if_set
from .base import BaseFunction, api_function, resolve_fields

__all__ = ("KeypairResourcePolicy",)

_default_list_fields = (
    keypair_resource_policy_fields["name"],
    keypair_resource_policy_fields["created_at"],
    keypair_resource_policy_fields["total_resource_slots"],
    keypair_resource_policy_fields["max_concurrent_sessions"],
    keypair_resource_policy_fields["idle_timeout"],
    keypair_resource_policy_fields["max_containers_per_session"],
    keypair_resource_policy_fields["allowed_vfolder_hosts"],
    keypair_resource_policy_fields["max_session_lifetime"],
    keypair_resource_policy_fields["max_pending_session_count"],
    keypair_resource_policy_fields["max_pending_session_resource_slots"],
    keypair_resource_policy_fields["max_concurrent_sftp_sessions"],
)

_default_detail_fields = (
    keypair_resource_policy_fields["name"],
    keypair_resource_policy_fields["created_at"],
    keypair_resource_policy_fields["total_resource_slots"],
    keypair_resource_policy_fields["max_concurrent_sessions"],
    keypair_resource_policy_fields["idle_timeout"],
    keypair_resource_policy_fields["max_containers_per_session"],
    keypair_resource_policy_fields["allowed_vfolder_hosts"],
    keypair_resource_policy_fields["max_session_lifetime"],
    keypair_resource_policy_fields["max_pending_session_count"],
    keypair_resource_policy_fields["max_pending_session_resource_slots"],
    keypair_resource_policy_fields["max_concurrent_sftp_sessions"],
)


class KeypairResourcePolicy(BaseFunction):
    """
    Provides interactions with keypair resource policy.
    """

    def __init__(self, access_key: str):
        self.access_key = access_key

    @api_function
    @classmethod
    async def create(
        cls,
        name: str,
        *,
        default_for_unspecified: str,
        total_resource_slots: str,
        max_session_lifetime: int,
        max_concurrent_sessions: int,
        max_concurrent_sftp_sessions: int,
        max_containers_per_session: int,
        idle_timeout: int,
        vfolder_host_perms: str | Undefined = undefined,
        max_pending_session_count: int | Undefined = undefined,
        max_pending_session_resource_slots: str | Undefined = undefined,
        fields: Iterable[FieldSpec | str] | None = None,
    ) -> dict:
        """
        Creates a new keypair resource policy with the given options.
        You need an admin privilege for this operation.
        """
        q = (
            "mutation($name: String!, $input: CreateKeyPairResourcePolicyInput!) {"
            + "  create_keypair_resource_policy(name: $name, props: $input) {"
            "    ok msg resource_policy { $fields }"
            "  }"
            "}"
        )
        resolved_fields = resolve_fields(
            fields, keypair_resource_policy_fields, (keypair_resource_policy_fields["name"],)
        )
        q = q.replace("$fields", " ".join(resolved_fields))
        inputs = {
            "default_for_unspecified": default_for_unspecified,
            "total_resource_slots": total_resource_slots,
            "max_session_lifetime": max_session_lifetime,
            "max_concurrent_sessions": max_concurrent_sessions,
            "max_concurrent_sftp_sessions": max_concurrent_sftp_sessions,
            "max_containers_per_session": max_containers_per_session,
            "idle_timeout": idle_timeout,
            "allowed_vfolder_hosts": vfolder_host_perms,
            "max_pending_session_count": max_pending_session_count,
            "max_pending_session_resource_slots": max_pending_session_resource_slots,
        }
        set_if_set(inputs, "allowed_vfolder_hosts", vfolder_host_perms)
        variables = {
            "name": name,
            "input": inputs,
        }
        data = await api_session.get().Admin._query(q, variables)
        return data["create_keypair_resource_policy"]

    @api_function
    @classmethod
    async def update(
        cls,
        name: str,
        *,
        default_for_unspecified: int | Undefined = undefined,
        max_session_lifetime: int | Undefined = undefined,
        max_concurrent_sessions: int | Undefined = undefined,
        max_concurrent_sftp_sessions: int | Undefined = undefined,
        max_containers_per_session: int | Undefined = undefined,
        idle_timeout: int | Undefined = undefined,
        total_resource_slots: str | Undefined = undefined,
        vfolder_host_perms: str | Undefined = undefined,
        max_pending_session_count: int | Undefined = undefined,
        max_pending_session_resource_slots: str | Undefined = undefined,
    ) -> dict:
        """
        Updates an existing keypair resource policy with the given options.
        You need an admin privilege for this operation.
        """
        q = (
            "mutation($name: String!, $input: ModifyKeyPairResourcePolicyInput!) {"
            + "  modify_keypair_resource_policy(name: $name, props: $input) {    ok msg  }}"
        )
        inputs: dict[str, Any] = {}
        set_if_set(inputs, "default_for_unspecified", default_for_unspecified)
        set_if_set(inputs, "total_resource_slots", total_resource_slots)
        set_if_set(inputs, "max_session_lifetime", max_session_lifetime)
        set_if_set(inputs, "max_concurrent_sessions", max_concurrent_sessions)
        set_if_set(inputs, "max_concurrent_sftp_sessions", max_concurrent_sftp_sessions)
        set_if_set(inputs, "max_containers_per_session", max_containers_per_session)
        set_if_set(inputs, "idle_timeout", idle_timeout)
        set_if_set(inputs, "allowed_vfolder_hosts", vfolder_host_perms)
        set_if_set(inputs, "max_pending_session_count", max_pending_session_count)
        set_if_set(inputs, "max_pending_session_resource_slots", max_pending_session_resource_slots)
        variables = {
            "name": name,
            "input": inputs,
        }
        data = await api_session.get().Admin._query(q, variables)
        return data["modify_keypair_resource_policy"]

    @api_function
    @classmethod
    async def delete(cls, name: str) -> dict:
        """
        Deletes an existing keypair resource policy with given name.
        You need an admin privilege for this operation.
        """
        q = (
            "mutation($name: String!) {"
            + "  delete_keypair_resource_policy(name: $name) {    ok msg  }}"
        )
        variables = {
            "name": name,
        }
        data = await api_session.get().Admin._query(q, variables)
        return data["delete_keypair_resource_policy"]

    @api_function
    @classmethod
    async def list(
        cls,
        fields: Sequence[FieldSpec] = _default_list_fields,
    ) -> Sequence[dict]:
        """
        Lists the keypair resource policies.
        You need an admin privilege for this operation.
        """
        q = "query {  keypair_resource_policies {    $fields  }}"
        q = q.replace("$fields", " ".join(f.field_ref for f in fields))
        data = await api_session.get().Admin._query(q)
        return data["keypair_resource_policies"]

    @api_function
    async def info(
        self,
        name: str,
        fields: Sequence[FieldSpec] = _default_detail_fields,
    ) -> dict:
        """
        Returns the resource policy's information.

        :param fields: Additional per-agent query fields to fetch.

        .. versionadded:: 19.03
        """
        q = "query($name: String) {  keypair_resource_policy(name: $name) {    $fields  }}"
        q = q.replace("$fields", " ".join(f.field_ref for f in fields))
        variables = {
            "name": name,
        }
        data = await api_session.get().Admin._query(q, variables)
        return data["keypair_resource_policy"]



from typing import Optional

from ..request import Request
from .base import BaseFunction, api_function

__all__ = ("Auth",)


class Auth(BaseFunction):
    """
    Provides the function interface for login session management and authorization.
    """

    @api_function
    @classmethod
    async def login(cls, user_id: str, password: str, otp: Optional[str] = None) -> dict:
        """
        Log-in into the endpoint with the given user ID and password.
        It creates a server-side web session and return
        a dictionary with ``"authenticated"`` boolean field and
        JSON-encoded raw cookie data.
        """
        rqst = Request("POST", "/server/login")
        body = {
            "username": user_id,
            "password": password,
        }
        if otp:
            body["otp"] = otp
        rqst.set_json(body)
        async with rqst.fetch(anonymous=True) as resp:
            data = await resp.json()
            data["cookies"] = resp.raw_response.cookies
            data["config"] = {
                "username": user_id,
            }
            return data

    @api_function
    @classmethod
    async def logout(cls) -> None:
        """
        Log-out from the endpoint.
        It clears the server-side web session.
        """
        rqst = Request("POST", "/server/logout")
        async with rqst.fetch() as resp:
            resp.raw_response.raise_for_status()

    @api_function
    @classmethod
    async def update_password(
        cls, old_password: str, new_password: str, new_password2: str
    ) -> dict:
        """
        Update user's password. This API works only for account owner.
        """
        rqst = Request("POST", "/auth/update-password")
        rqst.set_json({
            "old_password": old_password,
            "new_password": new_password,
            "new_password2": new_password2,
        })
        async with rqst.fetch() as resp:
            return await resp.json()

    @api_function
    @classmethod
    async def update_password_no_auth(
        cls, domain: str, user_id: str, current_password: str, new_password: str
    ) -> dict:
        """
        Update user's password. This is used to update `EXPIRED` password only.
        This function fetch a request to manager.
        """

        rqst = Request("POST", "/auth/update-password-no-auth")
        rqst.set_json({
            "domain": domain,
            "username": user_id,
            "current_password": current_password,
            "new_password": new_password,
        })
        async with rqst.fetch(anonymous=True) as resp:
            return await resp.json()

    @api_function
    @classmethod
    async def update_password_no_auth_in_session(
        cls, user_id: str, current_password: str, new_password: str
    ) -> dict:
        """
        Update user's password. This is used to update `EXPIRED` password only.
        This function fetch a request to webserver.
        """

        rqst = Request("POST", "/server/update-password-no-auth")
        rqst.set_json({
            "username": user_id,
            "current_password": current_password,
            "new_password": new_password,
        })
        async with rqst.fetch(anonymous=True) as resp:
            return await resp.json()
